
/* Chrysalide - Outil d'analyse de fichiers binaires
 * id.h - prototypes pour la constitution des identifiants de taille dynamique
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _DEBUG_JDWP_MISC_ID_H
#define _DEBUG_JDWP_MISC_ID_H


#include <stdbool.h>


#include "../jdwp_def.h"
#include "../../../arch/archbase.h"



/* Ecrit un identifiant enregistré dans une taille dynamique. */
bool _set_jdwp_dynsized_id(const jdwp_dynsized_id *, uint32_t, bin_t *, off_t *, off_t);


#define set_jdwp_object_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_thread_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_thread_group_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_string_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_class_loader_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_class_object_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_array_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_reference_type_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->object_id_size, blob, pos, len)

#define set_jdwp_class_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->reference_type_id_size, blob, pos, len)

#define set_jdwp_interface_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->reference_type_id_size, blob, pos, len)

#define set_jdwp_array_type_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->reference_type_id_size, blob, pos, len)

#define set_jdwp_method_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->method_id_size, blob, pos, len)

#define set_jdwp_field_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->field_id_size, blob, pos, len)

#define set_jdwp_frame_id(id, sizes, blob, pos, len) \
    _set_jdwp_dynsized_id(id, sizes->frame_id_size, blob, pos, len)


/* Lit un identifiant enregistré dans une taille dynamique. */
bool _get_jdwp_dynsized_id(const bin_t *, off_t *, off_t, uint32_t, jdwp_dynsized_id *);


#define get_jdwp_object_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_thread_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_thread_group_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_string_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_class_loader_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_class_object_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_array_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_reference_type_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->object_id_size, id)

#define get_jdwp_class_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->reference_type_id_size, id)

#define get_jdwp_interface_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->reference_type_id_size, id)

#define get_jdwp_array_type_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->reference_type_id_size, id)

#define get_jdwp_method_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->method_id_size, id)

#define get_jdwp_field_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->field_id_size, id)

#define get_jdwp_frame_id(blob, pos, len, sizes, id) \
    _get_jdwp_dynsized_id(blob, pos, len, sizes->frame_id_size, id)



#endif  /* _DEBUG_JDWP_MISC_ID_H */
